/*
 * Cr le 5 juin 07
 *
 * Pour changer le modle de ce fichier gnr, allez  :
 * Fentre&gt;Prfrences&gt;Java&gt;Gnration de code&gt;Code et commentaires
 */
package fr.gouv.finances.cp.xemelios.common.config;

import java.lang.reflect.Constructor;
import java.util.HashMap;

import org.apache.log4j.Logger;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

import fr.gouv.finances.cp.utils.xml.marshal.NoeudModifiable;
import fr.gouv.finances.cp.utils.xml.marshal.XmlAttributes;

public class DocumentsDeriveParser extends DefaultHandler {
	private static Logger logger = Logger.getLogger(DocumentsDeriveParser.class);
	
	NoeudModifiable current, oldCurrent = null;
	boolean inConstruction = false;
	String closeConstructionOn=null;
	String baseDirectory;
	private HashMap<String, Class> mapping = new HashMap<String, Class>();
	
	// la liste des configuration dj lues et dont on peut driver
	private DocumentsModel dmsReference = null;
	
	// le DocumentModel que l'on enrichi
	private DocumentModel dmEnrichi = null;
	
	public DocumentModel getDmEnrichi() {
		return dmEnrichi;
	}

	public DocumentsDeriveParser (DocumentsModel dm, String baseDir) {
		dmsReference = dm;
		baseDirectory=baseDir;
		
		mapping.put(ChampModel.TAG,ChampModel.class);
		mapping.put(CritereModel.TAG,CritereModel.class);
		mapping.put(DocumentModel.TAG,DocumentModel.class);
		mapping.put(ElementModel.TAG,ElementModel.class);
		mapping.put(EtatModel.TAG,EtatModel.class);
		mapping.put(InputModel.TAG, InputModel.class);
		mapping.put(ListeResultatModel.TAG,ListeResultatModel.class);
		mapping.put(OptionModel.TAG,OptionModel.class);
		mapping.put(SelectModel.TAG,SelectModel.class);
		mapping.put(EnteteModel.TAG,EnteteModel.class);
		mapping.put(PropertyModel.TAG,PropertyModel.class);
		mapping.put(PluginModel.TAG,PluginModel.class);
		mapping.put(SpecialKeyModel.TAG,SpecialKeyModel.class);
		mapping.put(SpecialKeyEntryModel.TAG,SpecialKeyEntryModel.class);
		mapping.put(BlankModel.TAG,BlankModel.class);
		mapping.put(NamespaceModel.TAG,NamespaceModel.class);
		mapping.put(NSModel.TAG,NSModel.class);
		mapping.put("recherche-paire",RecherchePaireModel.class);
		mapping.put("collectivite-path",RecherchePaireModel.class);
		mapping.put("budget-path",RecherchePaireModel.class);
		mapping.put("valeur",XPathModel.class);
		mapping.put("path",XPathModel.class);
		mapping.put("code-path",XPathModel.class);
		mapping.put("libelle-path",XPathModel.class);
		mapping.put("libelles",IgnoredModel.class);
		mapping.put("tag",IgnoredModel.class);
		mapping.put("enum",IgnoredModel.class);
		mapping.put("champ-optionnel",ChampModel.class);
		mapping.put("default-budget",OptionModel.class);
		mapping.put("desc",OptionModel.class);
		mapping.put("header",OptionModel.class);
		mapping.put("liste-export",ListeResultatModel.class);
		mapping.put("key1",TextModel.class);
		mapping.put("key2",TextModel.class);
		mapping.put(ReferenceModel.TAG,ReferenceModel.class);
		mapping.put(VariableModel.TAG,VariableModel.class);
		mapping.put(EnvironmentModel.TAG,EnvironmentModel.class);
		mapping.put("libelle",TextModel.class);
		mapping.put(HiddenModel.TAG,HiddenModel.class);
		mapping.put(ParametersModel.TAG,ParametersModel.class);
		mapping.put(ParameterModel.TAG,ParameterModel.class);		
		mapping.put("parent",SimpleElement.class);
		mapping.put("enfants",EnfantModel.class);
		mapping.put("enfant",SimpleElement.class);
	}

	@Override
	public void startElement(String uri, String localName, String qName, Attributes attributes) throws SAXException {
		/**
		 * Si on lit un dbut de balise documents-derives, on l'ignore
		 */
		if ("documents-derives".equals(localName)) return;
		
		/**
		 * Si on lit un dbut de balise document, il faut aller chercher la configuration dont on drive via l'attribut "extends"
		 */
		if ("document".equals(localName)) {
			if (dmsReference == null || dmsReference.getDocuments().size()==0) throw new SAXException ("Aucun document n'est disponible pour hritage.");
            String extendsId = attributes.getValue("extends");
            DocumentModel dmEtendu = dmsReference.getDocumentById(extendsId);
            if(dmEtendu==null) throw new SAXException("Configuration de "+extendsId+" introuvable");
			dmEnrichi = dmEtendu.clone();
			// on le prends comme point de dpart.
			current = dmEnrichi;
			// on mets  jour 
			current.modifyAttrs(attributes);
			dmEnrichi.setBaseDirectory(baseDirectory);
			dmEnrichi.setExtendedDocId(extendsId);
			/**
			 * ATTENTION
			 * Si la configuration dont on drive est une configuration drive, il se peut qu'elle ne soit pas encore charge.
			 * La drivation de plusieurs niveaux n'est pas prise en charge...
			 */
			
			
		/**
		 * Sinon on tente d'enrichir le modle trouv
		 */	
		} else {
			
			/**
			 * On rcupre dans la configuration dont on drive, le noeud qui nous intresse, c'est  dire le noeud qui doit tre
			 * modifi par la nouvelle configuration 
			 */
			
			NoeudModifiable child = null;
			// la clef du fils cherch.
			// Elle est soit simplement la valeur d'un attribut,
			// soit la concatnation de plusieurs
			String clef = "";
			Object ob = current.getChildIdAttrName(localName);
			if (ob!=null) {
				String[] clefs = (String[])ob;
				for (String c:clefs) {
					clef += attributes.getValue(c)+"|";
				}
				if (clef.endsWith("|")) clef = clef.substring(0, clef.length()-1);
			}
			child = current.getChildAsNoeudModifiable(localName,clef);
						
			/**
			 * Si ce noeud n'existe pas, il s'agit alors d'un noeud  crer
			 * Si on est en "construction", cela veux dire que l'on a commenc  crer un nouveau fils qui a lui mme des fils. 
			 * On se trouve par exemple  traiter une valeur d'un nouveau champs :
			 * 	<champ id="lr8" datatype="string" libelle="Montant Cumul annuel" affichable="true" exportable="true" identifiant="false">
	         *  	<valeur>BlocBordereau/MtCumulAnnuel/@V</valeur>
	         *  </champ>
			 * 
			 */
			if(child==null || inConstruction) {
				try {
					Class[] paramTypes = new Class[1]; paramTypes[0] = String.class;
					Class clazz = mapping.get (localName);
					Constructor ctr = clazz.getConstructor(paramTypes);
					Object[] params = new Object[1]; params[0] = localName;
					NoeudModifiable nouveau = (NoeudModifiable)ctr.newInstance(params);
					nouveau.getAttributes(new XmlAttributes(attributes));
					current.addChild(nouveau, localName);
					current=nouveau;
					closeConstructionOn=localName;
					inConstruction = true;
				} catch (Exception e) {
					logger.error ("Erreur lors de la cration du noeud ("+localName+") : ", e);
					throw new SAXException(e);
				}
				
			/**
			 * Sinon, on modifie simplement les attributs du noeud avec ceux ventuellement redfinis et on se repositionne pour la suite
			 */
			} else {
				current = child;
				current.modifyAttrs(attributes);
			}
		}
	}

	@Override
	public void endElement(String uri, String localName, String qName) throws SAXException {
		/**
		 * Si on lit une fin de balise documents-derives, on l'ignore
		 */
		if ("documents-derives".equals(localName)) return;
		
		/**
		 * Si l'on a une fin de document, alors on a fini d'enrichir dmEnrichi. Il faut l'ajouter  la liste des docs
		 */
		if ("document".equals(localName)) {
			dmsReference.addChild(dmEnrichi, DocumentModel.TAG);
		} 
		/**
		 * On a trait un noeud. On se repositionne pour la suite.
		 */
		else {
			current = current.getParentAsNoeudModifiable();
			if(inConstruction && localName.equals(closeConstructionOn)) inConstruction = false;
		}
	}

	@Override
	public void characters(char[] ch, int start, int length) throws SAXException {
		if (current != null) {
			// Attention, on supprime l'ancienne valeur de ce tag; 
			// peut tre existe-t-il des cas o cela ne doit pas tre fait
			if (!current.equals(oldCurrent)) {
				current.resetCharData();
				oldCurrent = current;
			}
			current.addCharacterData(new String (ch, start, length));
		}
		else
			super.characters(ch, start, length);
	} 
	
	
}
