/*
 * Copyright
 *   2007 axYus - www.axyus.com
 *   2007 C.Marchand - christophe.marchand@axyus.com
 *
 * This file is part of XEMELIOS.
 *
 * XEMELIOS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * XEMELIOS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with XEMELIOS; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package fr.gouv.finances.cp.xemelios.data.impl.sqlconfig;

import org.xml.sax.SAXException;

import fr.gouv.finances.cp.utils.xml.marshal.InvalidXmlDefinition;
import fr.gouv.finances.cp.utils.xml.marshal.XmlAttributes;
import fr.gouv.finances.cp.utils.xml.marshal.XmlMarshallable;
import fr.gouv.finances.cp.utils.xml.marshal.XmlOutputter;

/**
 *
 * @author chm
 */
public class TSql implements XmlMarshallable {
    public static final String TAG = "sql";
    private boolean failOnError = true;
    private StringBuffer data;
    private TPatch parent;
    
    /** Creates a new instance of TSql */
    public TSql(String tagName) {
        super();
        data = new StringBuffer();
    }

    public void addCharacterData(String cData) throws SAXException {
        data.append(cData);
    }

    public XmlMarshallable getAttributes(XmlAttributes attributes) throws SAXException {
        if(attributes.getValue("failonerror")!=null)
            failOnError = attributes.getBooleanValue("failonerror");
        return this;
    }

    public void addChild(XmlMarshallable child, String tagName) throws SAXException { }

    public void marshall(XmlOutputter output) {
        output.startTag(TAG);
        if(!failOnError) output.addAttribute("failonerror",failOnError);
        output.addCharacterData(data.toString());
        output.endTag(TAG);
    }

    public void validate() throws InvalidXmlDefinition {
    }
    
    public String getSql() {
        return data.toString().replaceAll("\\$\\{base-name\\}",getParent().getParent().getBaseName());
    }
    public TPatch getParent() { return parent; }
    public void setParent(TPatch parent) { this.parent = parent; }
    public TSql clone() {
        TSql other = new TSql(TAG);
        other.data.append(this.data.toString());
        other.failOnError=this.failOnError;
//        other.parent = this.parent.clone();
        return other;
    }
    public TSql clone(TPatch parent) {
        TSql other = clone();
        other.parent=parent;
        return other;
    }
    public boolean isFailOnError() { return failOnError; }
}
