/*
 * Copyright 
 *   2005 axYus - www.axyus.com
 *   2005 C.Marchand - christophe.marchand@axyus.com
 * 
 * This file is part of XEMELIOS.
 * 
 * XEMELIOS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * XEMELIOS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with XEMELIOS; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package fr.gouv.finances.cp.xemelios.updater.config;

import org.xml.sax.SAXException;

import fr.gouv.finances.cp.utils.xml.marshal.InvalidXmlDefinition;
import fr.gouv.finances.cp.utils.xml.marshal.XmlAttributes;
import fr.gouv.finances.cp.utils.xml.marshal.XmlMarshallable;
import fr.gouv.finances.cp.utils.xml.marshal.XmlOutputter;

public class XemeliosUpdateModel implements XmlMarshallable {
	public static final transient String TAG = "xemelios-update";
	
	private ReleaseModel release = null;
	private TexteModel originalLocation = null;
	private UpdatePartModel xemeliosUpdates, configUpdates, featuresUpdates;
	private ForeignSitesModel foreignSites;

	public XemeliosUpdateModel(String tag) {
		super();
	}

	public void addCharacterData(String cData) throws SAXException {
	}

	public void addChild(XmlMarshallable child, String tagName) throws SAXException {
		if(ReleaseModel.TAG.equals(tagName)) release = (ReleaseModel)child;
		else if("original-location".equals(tagName)) originalLocation = (TexteModel)child;
		else if("xemelios-updates".equals(tagName)) xemeliosUpdates = (UpdatePartModel)child;
		else if("config-updates".equals(tagName)) configUpdates = (UpdatePartModel)child;
		else if("features-updates".equals(tagName)) featuresUpdates = (UpdatePartModel)child;
		else if("foreign-sites".equals(tagName)) foreignSites = (ForeignSitesModel)child;
	}

	public XmlMarshallable getAttributes(XmlAttributes attributes) throws SAXException {
		return this;
	}

	public void marshall(XmlOutputter output) {
		output.startTag(TAG);
		if(release!=null) release.marshall(output);
		if(originalLocation!=null) originalLocation.marshall(output);
		if(xemeliosUpdates!=null) xemeliosUpdates.marshall(output);
		if(configUpdates!=null) configUpdates.marshall(output);
		if(featuresUpdates!=null) featuresUpdates.marshall(output);
		if(foreignSites!=null) foreignSites.marshall(output);
		output.endTag(TAG);
	}

	public void validate() throws InvalidXmlDefinition {
		if(release==null) throw new InvalidXmlDefinition("element release is mandatory for element "+TAG);
	}
	public XemeliosUpdateModel clone() {
		XemeliosUpdateModel xu = new XemeliosUpdateModel(TAG);
		xu.release=release.clone();
		if(originalLocation!=null) xu.originalLocation=originalLocation.clone();
		if(xemeliosUpdates!=null) xu.xemeliosUpdates=xemeliosUpdates.clone();
		if(configUpdates!=null) xu.configUpdates=configUpdates.clone();
		if(featuresUpdates!=null) xu.featuresUpdates=featuresUpdates.clone();
		if(foreignSites!=null) xu.foreignSites=foreignSites.clone();
		return xu;
	}

	public UpdatePartModel getConfigUpdates() {
		return configUpdates;
	}

	public void setConfigUpdates(UpdatePartModel configUpdates) {
		this.configUpdates = configUpdates;
	}

	public UpdatePartModel getFeaturesUpdates() {
		return featuresUpdates;
	}

	public void setFeaturesUpdates(UpdatePartModel featuresUpdates) {
		this.featuresUpdates = featuresUpdates;
	}

	public ForeignSitesModel getForeignSites() {
		return foreignSites;
	}

	public void setForeignSites(ForeignSitesModel foreignSites) {
		this.foreignSites = foreignSites;
	}

	public TexteModel getOriginalLocation() {
		return originalLocation;
	}

	public void setOriginalLocation(TexteModel originalLocation) {
		this.originalLocation = originalLocation;
	}

	public ReleaseModel getRelease() {
		return release;
	}

	public void setRelease(ReleaseModel release) {
		this.release = release;
	}

	public UpdatePartModel getXemeliosUpdates() {
		return xemeliosUpdates;
	}

	public void setXemeliosUpdates(UpdatePartModel xemeliosUpdates) {
		this.xemeliosUpdates = xemeliosUpdates;
	}
}
